#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Completion profiling
# 
# Profiling avec [cProfile](https://docs.python.org/3.7/library/profile.html), [memory_profiler](https://pypi.org/project/memory-profiler/), [pyinstrument](https://github.com/joerick/pyinstrument), [snakeviz](https://jiffyclub.github.io/snakeviz/).
# 
# [line_profiler](https://github.com/rkern/line_profiler) ne semble pas plus être maintenu.

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')
import matplotlib.pyplot as plt
plt.style.use('ggplot')
from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Setup

# ### Function to profile

# In[2]:


from mlstatpy.nlp.completion import CompletionTrieNode

def gain_dynamique_moyen_par_mot(queries, weights):
    per = list(zip(weights, queries))
    total = sum(weights) * 1.0
    res = []
    trie = CompletionTrieNode.build([(None, q) for _, q in per])
    trie.precompute_stat()
    trie.update_stat_dynamic()
    wks = [(w, p, len(w) - trie.min_keystroke0(w)[0]) for p, w in per]
    wks_dyn = [(w, p, len(w) - trie.min_dynamic_keystroke(w)[0])
               for p, w in per]
    wks_dyn2 = [(w, p, len(w) - trie.min_dynamic_keystroke2(w)[0])
                for p, w in per]
    gain = sum(g * p / total for w, p, g in wks)
    gain_dyn = sum(g * p / total for w, p, g in wks_dyn)
    gain_dyn2 = sum(g * p / total for w, p, g in wks_dyn2)
    ave_length = sum(len(w) * p / total for p, w in per)
    return gain, gain_dyn, gain_dyn2, ave_length


# ### Data

# In[3]:


from mlstatpy.data.wikipedia import download_titles
file_titles = download_titles(country='fr')


# In[4]:


len(file_titles)


# In[5]:


from mlstatpy.data.wikipedia import enumerate_titles
list_titles = list(sorted(set(_ for _ in enumerate_titles(file_titles) if 'A' <= _[0] <= 'Z')))


# In[6]:


import random
sample1000 = random.sample(list_titles, 1000)
with open("sample1000.txt", "w", encoding="utf-8") as f:
    f.write("\n".join(sample1000))


# ## Standard modules

# ### cProfile

# In[7]:


import cProfile, io, pstats, os

def toprofile0(lines):
    gain_dynamique_moyen_par_mot(lines, [1.0] * len(lines))

def doprofile(lines, filename):
    pr = cProfile.Profile()
    pr.enable()
    toprofile0(lines)
    pr.disable()
    s = io.StringIO()
    ps = pstats.Stats(pr, stream=s).sort_stats('cumulative')
    ps.print_stats()
    rem = os.path.normpath(os.path.join(os.getcwd(), "..", "..", ".."))
    res = s.getvalue().replace(rem, "")
    ps.dump_stats(filename)
    return res


# In[8]:


r = doprofile(sample1000, "completion.prof")
print(r)


# ## Others informations when profiling

# ### memory_profiler
# 
# See [memory_profiler](https://pypi.python.org/pypi/memory_profiler/0.41). Version 0.56 is bugged (see [#258](https://github.com/pythonprofilers/memory_profiler/issues/258)).

# In[9]:


from memory_profiler import profile, __version__
get_ipython().run_line_magic('load_ext', 'memory_profiler')


# In[10]:


get_ipython().run_line_magic('memit', 'toprofile0(sample1000)')


# In[11]:


from io import StringIO
st = StringIO()
@profile(stream=st)
def toprofile(lines):
    gain_dynamique_moyen_par_mot(lines, [1.0] * len(lines))
toprofile(sample1000)


# In[12]:


get_ipython().run_cell_magic('file', 'temp_mem_profile.py', '\nfrom mlstatpy.nlp.completion import CompletionTrieNode\nfrom memory_profiler import profile\n\n@profile(precision=4)\ndef gain_dynamique_moyen_par_mot(queries, weights):\n    per = list(zip(weights, queries))\n    total = sum(weights) * 1.0\n    res = []\n    trie = CompletionTrieNode.build([(None, q) for _, q in per])\n    trie.precompute_stat()\n    trie.update_stat_dynamic()\n    wks = [(w, p, len(w) - trie.min_keystroke0(w)[0]) for p, w in per]\n    wks_dyn = [(w, p, len(w) - trie.min_dynamic_keystroke(w)[0])\n               for p, w in per]\n    wks_dyn2 = [(w, p, len(w) - trie.min_dynamic_keystroke2(w)[0])\n                for p, w in per]\n    gain = sum(g * p / total for w, p, g in wks)\n    gain_dyn = sum(g * p / total for w, p, g in wks_dyn)\n    gain_dyn2 = sum(g * p / total for w, p, g in wks_dyn2)\n    ave_length = sum(len(w) * p / total for p, w in per)\n    return gain, gain_dyn, gain_dyn2, ave_length\n\n@profile(precision=4)\ndef toprofile():\n    with open("sample1000.txt", "r", encoding="utf-8") as f:\n        lines = [_.strip("\\n\\r ") for _ in f.readlines()]\n    gain_dynamique_moyen_par_mot(lines, [1.0] * len(lines))\ntoprofile()\n')


# In[13]:


import sys
cmd = sys.executable
from pyquickhelper.loghelper import run_cmd
cmd += " -m memory_profiler temp_mem_profile.py"
out, err = run_cmd(cmd, wait=True)
print(out)


# ## Static Visualization

# ### pyinstrument
# 
# See [pyinstrument](https://github.com/joerick/pyinstrument).

# In[14]:


from pyinstrument import Profiler

profiler = Profiler(use_signal=False)
profiler.start()

toprofile0(sample1000)

profiler.stop()
out = profiler.output_text(unicode=False, color=False)
print(out.replace("\\", "/"))


# ## Javascript Visualization

# ### SnakeViz

# In[15]:


get_ipython().run_line_magic('load_ext', 'snakeviz')


# L'instruction qui suit lance l'explorateur par défaut avec les données du profilage.

# In[16]:


# %snakeviz toprofile0(sample1000)


# In[17]:


from pyquickhelper.helpgen import NbImage
NbImage("images/func_info.jpg", width=400)


# ### vprof, py-spy
# 
# See [vprof](https://github.com/nvdv/vprof) or [py-spy](https://github.com/benfred/py-spy). The second one outputs a SVG file easy to handle.

# In[18]:


# from vprof import profiler

# needs to be run from a file not from a notebook
# profiler.run(toprofile0, 'cmh', args=(sample1000,), host='localhost', port=8000)


# In[19]:


from pyquickhelper.helpgen import NbImage
NbImage("images/vprof.jpg", width=800)


# In[20]: